//
// Created by daniel on 26.5.20.
//

#include "filemanager.h"

FileManager::FileManager()
{
}

FileManager::~FileManager()
{
    for(auto & model : m_Models)
        delete(model);
}

Model * FileManager::GetModel(size_t modelIndex)
{
    if( 0<=modelIndex && modelIndex < m_Models.size())
        return m_Models[modelIndex];
    return nullptr;
}

bool FileManager::Read(const std::string &fileName)
{
    std::ifstream input(fileName, std::ios::binary);
    if ( !input.is_open() )
        return false;
    if (!input.good()) {
        input.close();
        return false;
    }

    Process(input);
    return true;
}

void FileManager::Print() const {
    for(auto const & model : m_Models)
        std::cout << *model << std::endl;
}

bool FileManager::Process(std::ifstream & data)
{
    m_Models.clear();
    std::string line, word;

    while ( std::getline(data, line) )
    {
        Model * model = new Model();
        std::vector<std::string> gates;


        model->Start();
        while( true )
        {
            replace(line.begin(), line.end(), '\n', ' ');
            replace(line.begin(), line.end(), '\t', ' ');

            std::istringstream iline(line);


            while ( std::getline(iline, word, ' ' ) ) {
                if ( word == ".model" ) {
                    model->SetName( (std::getline(iline, word, ' ')) ? word : "");
                } else if ( word == ".names") {
                    gates.clear();
                    while (std::getline(iline, word, ' ') ) {
                        if (word != "") {
                            model->AddCommand(new LogicGate(word));
                            gates.push_back(word);
                        }
                    }
                    size_t size = gates.size();
                    for(size_t i=0; i<size-1; ++i)
                        model->AddConnection(gates[i], gates[size-1]);
                } else if ( word == ".latch") {
                    std::vector<std::string> params;
                    while ( std::getline(iline, word, ' ') ) {
                        if (word != "" )
                            params.push_back(word);
                    }
                    model->AddCommand(new LogicGate(params[0]));
                    model->SetOutput(params[0]);
                    model->AddCommand(new LogicGate(params[1]));
                } else if ( word == ".inputs" ) {
                    while ( std::getline(iline, word) ) {
                        while (std::getline(iline, word, ' ') ) {
                            if (word != "") {
                                model->AddCommand(new LogicGate(word));
                            }
                        }
                    }
                } else if ( word == ".outputs") {
                    while (std::getline(iline, word, ' ') ) {
                        if (word != "") {
                            model->AddCommand(new LogicGate(word));
                            model->SetOutput(word);
                        }
                    }
                } else if ( word == ".clock") {
                    while ( std::getline(iline, word) ) {}
                } else if ( word == ".end") {
                    break;
                } else if ( word[0] == '0' || word[0]=='1' || word[0]=='-') {
                    std::string input = word;
                    std::vector<std::string> params;
                    while ( std::getline(iline, word) ) {
                        if ( word != "" )
                            params.push_back(word);
                    }

                    if ( params.size() > 1 )
                        return false;
                    std::string output = ( ! params.empty() ) ? word : input;
                    std::string name = gates[gates.size()-1];
                    model->GetCommand( name )->AddSimunation( input, output );
                } else {
                    std::cout << "Wrong input." << "[" <<word << "]" << std::endl;
                    return false;
               }
           }
           if (word == ".end")
               break;
           if (!std::getline(data, line))
               return false;
           replace(line.begin(), line.end(), '\n', ' ');
           replace(line.begin(), line.end(), '\t', ' ');
       }
       model->Stop();
       m_Models.push_back(model);
       model = nullptr;
   }
   return true;
}
