//
// Created by Admin on 7/14/2020.
//

#include "generator.h"

Generator::Generator()
{
    std::srand(time(nullptr));
}

void Generator::GenerateIllinois(const Model * model, const std::string & configFile, std::ostream & output) const {
    std::vector<std::string> inputs;
    std::vector<std::string> outputs;
    const std::string INPUT = "SI";
    const std::string TEST = "TEST";
    const std::string OUTPUT = "SO";

    Config config;
    config.Load(configFile);

    for(size_t i=0; i < config.GetMax(); ++i)
    {
        std::string in = "SC"+ std::to_string(i);
        inputs.push_back( in );
    }

    for(auto const & out : model->GetOutputs())
        outputs.push_back( out->GetName());

    if ( model->GetName() != "" )
        output << ".model " << model->GetName() << "\n";

    if ( !inputs.empty() ) {
        output << ".inputs";
        for(auto const & in : inputs)
            output << " " << in;
        output << " " << INPUT << " " << TEST;
        output << "\n";
    }

    if ( !outputs.empty() ) {
        output << ".outputs";
        for(auto const & out : outputs)
            output << " O" << out;
        output << " " << OUTPUT;
        output << "\n";
    }
    output << "\n";
    for (auto const & in : model->GetInputs())
        output << ".latch I" << in->GetName() << " O" << in->GetName() << "\n";
    output << "\n";
    for (auto const & out : model->GetOutputs())
        output << ".latch I" << out->GetName() << " O" << out->GetName() << "\n";
    output << "\n";

    std::string last_input = "";
    for( auto const & scanchain : config.GetScanChains() )
    {
        for(size_t i=0; i<scanchain.GetScanCells().size(); ++i)
        {
            std::string name = scanchain.GetScanCells()[i];
            if ( i==0 )
                output << ".names " << INPUT << " " << inputs[i] <<  " " << TEST << " I" << name << "\n";
            else
                output << ".names " << scanchain.GetScanCells()[i-1] << " " << inputs[i] << " " << TEST << " I" << name << "\n";
            output << "1-0 1" << "\n-11 1" << "\n";


            if ( i == scanchain.GetScanCells().size()-1 ) {
                output << ".names " << INPUT << " O" << name << " " << TEST << " " << name << "\n";
                output << "1-0 1" << "\n-11 1" << "\n";
                last_input = name;
            }
            else
            {
                output << ".names O" << name << " " << name << "\n";
                output << "1 1\n";
            }
        }
        output << "\n";
    }

    for( size_t i=0; i<outputs.size(); ++i )
    {
        if ( i==(outputs.size()-1) )
            output << ".names " << outputs[i] << " " << last_input << " " << TEST << " I" << outputs[i] << "\n";
        else
            output << ".names " << outputs[i] << " O" << outputs[i+1] << " " << TEST << " I" << outputs[i] << "\n";
        output << "1-0 1" << "\n-11 1" << "\n";
    }
    if ( !outputs.empty() )
    {
        output << ".names O" << outputs[0]  << " " << outputs[0] << " " << TEST << " " << OUTPUT << "\n";
        output << "1-1 1\n-01 1\n";
        output << "\n";
    }
    GenerateNames(model, output);
}


void Generator::GenerateConfig(const std::vector<Command*> & inputs, std::ostream & os, size_t number, ScanChainsGenerateMode mode) const
{
	if (number > inputs.size())
		return;

    std::vector<std::string> cells;
    for(auto const & input : inputs)
        cells.push_back(input->GetName());

    std::random_shuffle(cells.begin(), cells.end(), Randomize);

    if ( mode == SCANCHAIN_NUMBER )
    {
    	if ( number > cells.size() )
    	{
 //   		std::cout << "Invalid number of scan-chain. <MAX " << cells.size() << ", YOURS " << number << ">" << std::endl;
    		return;
    	}

        size_t size = cells.size() / number;
        size_t modulo = cells.size() % number;
        size_t power_rows = modulo * (size+1); 
        
        size_t index = 0;
        for(size_t i=1; i<=power_rows; ++i)
        {
        	os << cells[i-1];
        	os << ((i % (size+1) == 0) ? "\n" : " ");
        }
        for(size_t i=1 + power_rows; i<=cells.size(); ++i)
        {
        	os << cells[i-1];
        	index++;
        	os << ((((i-power_rows) % size) == 0) ? "\n" : " ");
        }
    }
    else if ( mode == SCANCHAIN_SIZE)
    {
    	if ( number > cells.size() )
    	{
    		std::cout << "Invalid size of scan-chain. <MAX " << cells.size() << ", YOURS " << number << ">" << std::endl;
    		return;
    	}
    
        size_t size = number;
        for(size_t i=1; i<=cells.size(); ++i)
        {
            os << cells[i-1];
            os << (( (i % size == 0) || (i == cells.size()) ) ? "\n" : " ");
        }
    }
    else
    {
        size_t index = 0;
        int size = cells.size();
        while ( size > 0 )
        {
            int random = 1 + Randomize( number );
            if ( size - random < 0)
                random = size;
            for(int i=0; i<random; ++i)
                os << cells[index + i] << " ";
            os << "\n";
            index += random;
            size -= random;
        }
    }

}






void Generator::GenerateIllinoisTest(const Model * model, const std::string & configFile, std::ostream &output) const
{
    Config config;
    config.Load(configFile);


    if ( model->GetName() != "" )
        output << ".model " << model->GetName() << "\n";

    if ( !model->GetInputs().empty() ) {
        output << ".inputs";

        for(size_t i=0; i<config.GetMax(); ++i)
            output << " SC" << i;
        output << "\n";
    }

    if ( !model->GetOutputs().empty() ) {
        output << ".outputs";
        for(auto const & out : model->GetOutputs())
            output << " " << out->GetName();
        output << "\n";
    }

    output << "\n";
    for(auto const & scanChain : config.GetScanChains())
        GenerateScanChain(output,  scanChain);
    output << "\n";
    GenerateNames(model, output);
}

void Generator::GenerateCombinationalLogic(const Model * model, const std::string & fileName) const {
    std::ofstream output;
    output.open(fileName, std::ios::out | std::ios::app);

    if ( !CheckFile(output) )
        return;

    if ( model->GetName() != "" )
        output << ".model " << model->GetName() << "\n";

    if ( !model->GetInputs().empty() ) {
        output << ".inputs";
        for(auto const & input : model->GetInputs())
            output << " " << input->GetName();
        output << "\n";
    }

    if ( !model->GetOutputs().empty() ) {
        output << ".outputs";
        for(auto const & out : model->GetOutputs())
            output << " " << out->GetName();
        output << "\n";
    }
    GenerateNames(model, output);
    output.close();
}

int Generator::Randomize(int i)
{
    return std::rand()%i;
}

bool Generator::CheckFile(std::ofstream & file)
{
    if (!file.is_open())
        return false;
    if (!file.good()) {
        file.close();
        return false;
    }
    return true;
}



void Generator::GenerateScanChain(std::ostream & output, const ScanChain & scanChain) const
{
    for(size_t i=0; i < scanChain.GetScanCells().size(); ++i)
        output << ".names SC" << i << " " << scanChain.GetScanCells()[i] << "\n1 1\n";
}

void Generator::GenerateNames(const Model *model, std::ostream & output) const {
    if ( !model->GetCommands().empty() ) {
        for(auto const &command : model->GetCommands()) {
            if (!command.second->isInput()) {
                output << ".names";
                for(auto const & input : command.second->GetInputs())
                    output << " " << input->GetName();
                output << " " << command.first << "\n";
                output << command.second->GetDataSimulation();
            }
        }
    }
    output << ".end\n";
}

