//
// Created by daniel on 21.5.20.
//

#ifndef PROGRAM_CLOCKCONSTRAINT_H
#define PROGRAM_CLOCKCONSTRAINT_H

/*
 * --------------------------------------------
 * Blif file - Clock Constraints
 * --------------------------------------------
 *
 * A clock-constraint is used to set up the behavior of the simulated clocks, and to specify how clock events (rising
 * or falling edges) occur relative to one another. A clock-constraint is one or more of the following:
 *
 * .cycle <cycle-time>
 * .clock_event <event-percent> <event-1> [<event-2> ... <event-n>]
 *
 * cycle-time is a floating point number giving the clock cycle time for the model. It is a unitless number that is to
 * be interpreted by the user.
 *
 * event-percent is a floating point number representing a percentage of the clock cycle time at which a specific
 * .clock event occurs. Fifty percent is written as “50.0.”
 *
 * event-1 through event-n are one of the following:
 *
 * <rise-fall>’<clock-name>
 * (<rise-fall>’<clock-name> <before> <after>)
 *
 * where rise-fall is either “r” or “f” and stands for the rising or falling edge of the clock and clock-name is a
 * clock from the .clock construct. The apostrophe between rise-fall and clock-name is a seperator, and serves
 * no purpose in and of itself.
 *
 * before and after are floating point numbers in the same “units” as the cycle-time and are used to define the “skew”
 * in the clock edges. before represents maximum amount of time before the nominal time that the edge can
 * arrive; after represents the maximum amount of time after the nominal time that the edge can arrive. The
 * nominal time is event-percent% of the cycle-time. In the unparenthesized form for the clock-event, before
 * and after are assumed “0.0.”
 *
 * All events, event-1 ... event-n, specified in a single .clock event are to be linked together. A routine changing
 * any one edge should also modify the occurrence time of all the related clock edges.
 *
 * Example 1:
 * .clock clock1 clock2
 * .clock_event 50.0 r’clock1 (f’clock2 2.0 5.0)
 *
 * Example 2:
 * .clock clock1 clock2
 * .clock_event 50.0 r’clock1
 * .clock_event 50.0 (f’clock2 2.0 5.0)
 *
 * Both examples specify a nominal time of 50% of the cycle time, that the rising edge of clock1 must occur at
 * exactly the nominal time, and that the falling edge of clock2 may occur from 2.0 units before to 5.0 units after the
 * nominal time.
 *
 * In Example 1, if r’clock1 is later moved to a different nominal time by some routine then f’clock2 should also
 * be changed. However, in Example 2 changing r’clock1 would not affect f’clock2 even though they originally
 * have the same value of event-percent.
 *
 *
 */

class ClockConstraint {
public:

private:
};


#endif //PROGRAM_CLOCKCONSTRAINT_H
