//
// Created by daniel on 21.5.20.
//

#ifndef PROGRAM_FINITESTATEMACHINE_H
#define PROGRAM_FINITESTATEMACHINE_H

/*
 * --------------------------------------------
 * Blif file - Finite State Machine
 * --------------------------------------------
 *
 * A sequential circuit can be specified in BLIF logic form, as a finite state machine, or both. An fsm-description is
 * used to insert a finite state machine description of the current model. It is intended to represent the same sequential
 *
 * circuit as the current model (which contains logic), but in FSM form. The format of an fsm-description is:
 *
 * .start_kiss
 * .i <num-inputs>
 * .o <num-outputs>
 * [.p <num-terms>]
 * [.s <num-states>]
 * [.r <reset-state>]
 * <input> <current-state> <next-state> <output>
 * .
 * .
 * .
 * <input> <current-state> <next-state> <output>
 * .end_kiss
 * [.latch_order <latch-order-list>]
 * [<code-mapping>]
 *
 * num-inputs is the number of inputs to the FSM, which should agree with the number of inputs in the .inputs
 * construct for the current model.
 * 7num-outputs is the number of outputs of the FSM, which should agree with the number of outputs in the .outputs
 * construct for the current model.
 * num-terms is the number of “<input> <current-state> <next-state> <output>” 4-tuples that follow in the FSM
 * description.
 * num-states is the number of distinct states that appear in “<current-state>” and “<next-state>” columns.
 * reset-state is the symbolic name for the reset state for the FSM; it should appear somewhere in the “<current-
 * state>” column.
 * input is a sequence of num-inputs members of f 0, 1, – g .
 * output is a sequence of num-outputs members of f 0, 1, – g .
 * current-state and next-state are symbolic names for the current state and next state transitions of the FSM.
 * latch-order-list is a white-space-separated sequence of latch outputs.
 * code-mapping is newline separated sequence of:
 * .code <symbolic-name> <encoded-name>
 * num-terms and num-states do not have to be specified. If the reset-state is not given, it is assigned to be the
 * first state encountered in the “<current-state>” column.
 * The ordering of the bits in the input and output fields will be the same as the ordering of the variables in the
 * .inputs and .outputs constructs if both an fsm-description and logic functions are given.
 * latch-order-list and code-mapping are meant to be used when both an fsm-description and a logical description
 * of the model are given. The two constructs together provide a correspondence between the latches in the logical
 * description and the state variables in the fsm-description. In a code-mapping, symbolic-name consists of a symbolic
 * name from the “<current-state>” or “<next-state>” columns, and encoded-name is the pattern of bits ( f 0, 1 g ) that
 * represent the state encoding for symbolic-name. The code-mapping should only be given if both an fsm-description
 * and logic functions are given. .latch-order establishes a mapping between the bits of the encoded-names of the
 * code-mapping construct and the latches of the network. The order of the bits in the encoded names will be the
 * same as the order of the latch outputs in the latch-order-list. There should be the same number of bits in the
 * encoded-name as there are latches if both an fsm-description and a logical description are specified.
 * If both logic-gates and an fsm-description of the model are given, the logic-gate description of the model should
 * be consistent with the fsm-description, that is, they should describe the same circuit. If they are not consistent
 * there will be no sensible way to interpret the model, which should then cause an error to be returned.
 * If only the fsm-description of the network is given, it may be run through a state assignment routine and
 * given a logic implementation. A sole fsm-description, having no logic implementation, cannot be inserted into
 * another model by a model-reference; the state assigned network, or a network containing both logic-gates and an
 * fsm-description can.
 *
 * Example of an fsm-description:
 * .model 101
 * .start_kiss
 * .i 1
 * .o 1
 * 0 st0 st0 0
 * 1 st0 st1 0
 * 0 st1 st2 0
 * 1 st1 st1 0
 * 0 st2 st0 0
 * # outputs 1 whenever last 3 inputs were 1, 0, 1
 * 81 st2 st3 1
 * 0 st3 st2 0
 * 1 st3 st1 0
 * .end_kiss
 * .end
 * Above example with a consistent fsm-description and logical description:
 * .model
 * .inputs v0
 * .outputs v3.2
 * .latch
 * [6] v1
 * 0
 * .latch
 * [7] v2
 * 0
 * .start_kiss
 * .i 1
 * .o 1
 * .p 8
 * .s 4
 * .r st0
 * 0 st0 st0 0
 * 1 st0 st1 0
 * 0 st1 st2 0
 * 1 st1 st1 0
 * 0 st2 st0 0
 * 1 st2 st3 1
 * 0 st3 st2 0
 * 1 st3 st1 0
 * .end_kiss
 * .latch_order v1 v2
 * .code st0 00
 * .code st1 11
 * .code st2 01
 * .code st3 10
 * .names v0 [6]
 * 1 1
 * .names v0 v1 v2 [7]
 * -1- 1
 * 1-0 1
 * .names v0 v1 v2 v3.2
 * 101 1
 * .end
 *
 */

class FiniteStateMachine {
public:

private:
};

#endif //PROGRAM_FINITESTATEMACHINE_H
