//
// Created by daniel on 21.5.20.
//

#ifndef PROGRAM_SUBCIRCUIT_H
#define PROGRAM_SUBCIRCUIT_H

/*
 * --------------------------------------------
 * Blif file - Subcircuit (Model references)
 * --------------------------------------------
 * A model-reference is used to insert the logic functions of one model into the body of another. It is defined as
 * follows:
 *
 * .subckt <model-name> <formal-actual-list>
 *
 * model-name is a string giving the name of the model being inserted. It need not be previously defined in this file,
 * but should be defined somewhere in either this file, a .search file, or a master file that is .searching this file.
 * (see .search below)
 *
 * formal-actual-list is a mapping between the formal terminals (the decl-input-list, decl-output-list, and decl-clock-
 * list) of the called model model-name and the actual parameters of the current model. The actual parameters
 * may be any signals in the current model. The format for a formal-actual-list is the same as its format in a
 * library-gate.
 *
 * A .subckt construct can be viewed as creating a copy of the logic functions of the called model model-name,
 * including all of model-name’s generic-latches, in the calling model. The hierarchical nature of the BLIF description
 * of the model does not have to be preserved. Subcircuits can be nested, but cannot be self-referential or create a
 * cyclic dependency.
 *
 * Unlike a library-gate, a model-reference is not limited to one output.
 * The formals need not be specified in the same order as they are defined in the decl-input-list, decl-output-
 * list, or decl-clock-list; elements of the lists can be intermingled in any order, provided the names are given
 * correctly. Warning messages are printed if elements of the decl-input-list or decl-clock-list are not driven by an
 * actual parameter or if elements of the decl-output-list do not fan out to an actual parameter. Elements of the
 * decl-clock-list and decl-input-list may be driven by any logic function of the calling model.
 *
 * Example: rather than rewriting the entire BLIF description for a commonly used subcircuit several times, the
 * subcircuit can be described once and called as many times as necessary:
 *
 * .model 4bitadder
 * .inputs A3 A2 A1 A0 B3 B2 B1 B0 CIN
 * .outputs COUT S3 S2 S1 S0
 * .subckt fulladder a=A0 b=B0 cin=CIN
 * s=S0 cout=CARRY1
 * .subckt fulladder a=A3 b=B3 cin=CARRY3 s=S3 cout=COUT
 * .subckt fulladder b=B1 a=A1 cin=CARRY1 s=XX cout=CARRY2
 * .subckt fulladder a=JJ b=B2 cin=CARRY2 s=S2 cout=CARRY3
 * # for the sake of example,
 * .names XX S1
 * # formal output ‘s’ does not fanout to a primary output
 * 1 1
 * .names A2 JJ
 * # formal input ‘a’ does not fanin from a primary input
 * 1 1
 * 6.end
 *
 * .model fulladder
 * .inputs a b cin
 * .outputs s cout
 * .names a b k
 * 10 1
 * 01 1
 * .names k cin s
 * 10 1
 * 01 1
 * .names a b cin cout
 * 11- 1
 * 1-1 1
 * -11 1
 * .end
 *
 *
 */

class Subcircuit {
public:

private:
};

#endif //PROGRAM_SUBCIRCUIT_H
