//
// Created by daniel on 9.7.20.
//

#ifndef PROGRAM_SIMULATIONTREE_H
#define PROGRAM_SIMULATIONTREE_H

#include <iostream>

class Node {
public:
    Node(const char val,
         Node * zero,
         Node * one,
         Node * dash)
            : m_Val(val), m_Zero(zero), m_One(one), m_Dash(dash)
    {
    }
    virtual bool Empty() const {
        return false;
    }
    char m_Val;
    Node * m_Zero, *m_One, *m_Dash;
};

class EmptyNode : public Node {
public:
    EmptyNode()
    : Node(0, nullptr, nullptr, nullptr)
    {
    }
    bool Empty() const {
        return true;
    }
};

class SimulationTree {
public:

    SimulationTree()
    : m_Size(0),
      m_Positive(true),
      m_Root(new Node('*', nullptr, nullptr, nullptr) )
    {
    }

    ~SimulationTree()
    {
        DeleteTree(m_Root);
    }

    void Insert(const std::string &input, const bool positive = true) {
        m_Positive = positive;
        m_Simulation.insert(input);
        m_Root = Insert(m_Root, 0, input);
        m_Data += input + " " + ((positive) ? "1" : "0") + "\n";
    }

    std::string GetData() const {
        return m_Data;
    }

    char Find(const std::string &sequence) const {
        return ( FindSequence(m_Root, 0, sequence) && m_Positive ) ? '1' : '0';
    }

    void Print() const {
        size_t size = 0;
        for(auto const & input : m_Simulation) {
            std::cout << input << " " << m_Positive << std::endl;
            size = input.size();
        }
        std::cout << "------" << std::endl;


        PrintTree(m_Root, 0, size);
    }

private:

    void PrintTree(Node * node, size_t index, size_t size) const
    {

       if ( node )
       {
           std::cout << node->m_Val << "->";
           PrintTree(node->m_Zero, index+1, size);
           PrintTree(node->m_One , index+1, size);
           PrintTree(node->m_Dash, index+1, size);
       }

       if ( index == size && node )
            std::cout << std::endl;
    }

    void DeleteTree(Node * root)
    {
        if (root)
        {
            DeleteTree(root->m_Zero);
            DeleteTree(root->m_One);
            DeleteTree(root->m_Dash);
            delete(root);
        }
    }

    Node * Insert(Node * root, unsigned index, const std::string &sequence)
    {
        if (index == sequence.size())
            return root;
        if ( sequence[index]!='0' && sequence[index]!= '1' && sequence[index]!= '-' )
            throw "Invalid char.";

        if ( !root->Empty() ) {
            if (sequence[index] == '0') {
                if (!root->m_Zero)
                    root->m_Zero = (root->m_Zero) ? root->m_Zero : new Node('0', nullptr, nullptr, nullptr);
                root->m_Zero = Insert(root->m_Zero, index+1, sequence);
            } else if (sequence[index] == '1' ) {
                root->m_One = (root->m_One) ? root->m_One : new Node('1', nullptr, nullptr, nullptr);
                root->m_One = Insert(root->m_One, index+1, sequence);
            } else if (sequence[index] == '-' ) {
                root->m_Dash = (root->m_Dash) ? root->m_Dash : new Node('-', nullptr, nullptr, nullptr);
                root->m_Dash = Insert(root->m_Dash, index+1, sequence);
            }
        }
        return root;
    }

    bool FindSequence(Node * node, unsigned index, const std::string &sequence) const
    {
        if ( index == sequence.size() )
            return true;
        bool val = false;
        if ( sequence[index]=='0' && node->m_Zero )
            val = FindSequence(node->m_Zero, index+1, sequence);
        else if ( sequence[index]=='1' && node->m_One )
            val = FindSequence(node->m_One, index+1, sequence);
        if ( !val && node->m_Dash )
            val = FindSequence(node->m_Dash, index+1, sequence);
        return val;
    }

private:
    size_t m_Size;
    std::string m_Data;
    bool m_Positive;
    Node * m_Root;
    std::set<std::string> m_Simulation;

};

#endif //PROGRAM_SIMULATIONTREE_H
