//
// Created by daniel on 23.7.20.
//

#ifndef SIMULATOR_SLOADER_H
#define SIMULATOR_SLOADER_H


#include "../../global/loader/loader.h"

class SLoader : public Loader {
public:
    SLoader(int argc, char * argv[])
    : Loader(argc, argv),
      m_WriteInput(false)
    {
    }


    bool SetParam()
    {
        if ( m_Commands.size() < 4 )
            return false;

        m_Vectors.clear();
        if ( m_Commands[2]=="-vector" )
        {
            m_Vectors.push_back(m_Commands[3]);
        }
        else if ( m_Commands[2]=="-matrix" )
        {
            std::ifstream input(m_Commands[3]);

            if ( !input.is_open() )
                return false;
            if ( !input.good() )
            {
                input.close();
                return false;
            }

            std::string vector;
            while ( std::getline(input, vector) )
                m_Vectors.push_back(vector);
            input.close();
        }
        else
        {
            return false;
        }
        return true;
    }

    bool SetOutput()
    {
        if ( m_Commands.size() == 4 )
            return true;

        if ( m_Commands.size() == 5 && m_Commands[4]=="-input" )
        {
            m_WriteInput = true;
            return true;
        }


        if ( m_Commands.size() < 6 )
            return false;

        if ( m_Commands.size() >= 6)
        {
            if ( m_Commands[4] != "-name" )
                return false;
            m_FileOutput = m_Commands[5];
        }

        if ( m_Commands.size() == 7 )
        {
            if (m_Commands[6]=="-input")
                m_WriteInput = true;
            else
                return false;
        }
        else if (m_Commands.size() > 7)
        {
            return false;
        }
        return true;
    }


    bool Generate()
    {
        FileManager manager;

        if ( !manager.Read(m_FileInput) )
            return false;

        std::vector<std::string> outputs;
        if ( !manager.GetModel(0)->Simulation(m_Vectors, outputs) )
            return false;

        if ( m_FileOutput == "" )
        {
            if ( m_WriteInput )
            {
                for(size_t i=0; i<m_Vectors.size(); ++i)
                    std::cout << m_Vectors[i] << " " << outputs[i] << std::endl;
            }
            else
            {
                for(auto const & out : outputs)
                    std::cout << out << std::endl;
            }

        }
        else
        {
            std::ofstream output(m_FileOutput);
            if ( !Generator::CheckFile(output))
                return false;

            if ( m_WriteInput )
            {
                for(size_t i=0; i<m_Vectors.size(); ++i)
                    output << m_Vectors[i] << " " << outputs[i] << "\n";
            }
            else
            {
                for(auto const & out : outputs)
                    output << out << "\n";
            }

            output.close();
        }
        return true;
    }



private:
    std::vector<std::string> m_Vectors;
    bool m_WriteInput;
};


#endif //SIMULATOR_SLOADER_H
